@@ -16,6 +16,7 @@ import argparse
 import urllib.request
 from datetime import datetime
 import shutil
+import yaml
 
 import pandas as pd
 from jsonpath_ng import parse
@@ -233,7 +234,7 @@ def parse_fsh(fname_fsh: Path) -> Tuple[List[Dict], List[Dict]]:
     with open(fname_fsh, "r") as f:
         content = f.read()
 
-    re_group_capture = "[a-zA-Z0-9_-]+"
+    re_group_capture = r"[a-zA-Z0-9_\-\$]+"
 
     pattern = re.compile(
         rf"Profile: (?P<profile>{re_group_capture})[^\n]*\nParent: (?P<parent>{re_group_capture})[^\n]*\nId: (?P<id>{re_group_capture})",
@@ -270,12 +271,21 @@ def parse_fsh_generated(path: Path) -> Tuple[Dict, Dict, Dict, Dict, Dict]:
         }
 
     def parse_instance(fname: Path, json_data: str) -> Dict:
+        profile = parse("$.meta.profile").find(json_data)
+        resourceType = parse("$.resourceType").find(json_data)[0].value
+
+        if len(profile) == 0:
+            profile = resourceType
+        else:
+            profile = profile[0].value[0]
+
         return {
             parse("$.id")
             .find(json_data)[0]
             .value: {
                 "filename": fname.resolve(),
-                "profile": parse("$.meta.profile").find(json_data)[0].value[0],
+                "profile": profile,
+                "resourceType": resourceType,
             }
         }
 
@@ -310,6 +320,7 @@ def parse_fsh_generated(path: Path) -> Tuple[Dict, Dict, Dict, Dict, Dict]:
     for fname in path.glob("*.json"):
         json_data = json.load(open(fname))
         resourceType = parse("$.resourceType").find(json_data)[0].value
+
         if resourceType == "StructureDefinition":
             sd = parse_structure_definition(fname, json_data)
             sdefs.update(sd)
@@ -361,7 +372,11 @@ def get_fsh_base_path(path: Union[str, Path]) -> Path:
 
 
 def _validate_fsh_files(
-    path_output: Path, fnames: List[Path], fname_validator: str, verbose: bool = False
+    path_output: Path,
+    fnames: List[Path],
+    fname_validator: str,
+    fhir_version: str,
+    verbose: bool = False,
 ) -> List[ValidatorStatus]:
     """
     Validate FSH files.
@@ -373,6 +388,7 @@ def _validate_fsh_files(
     :param path_output: output path (of SUSHI project)
     :param fnames: FSH file names to validate (full paths)
     :param fname_validator: full path to FHIR Java validator file
+    :param fhir_version: FHIR version to use in validator
     :param verbose: Print more information
     :return: ValidatorStatus objects
     """
@@ -414,6 +430,7 @@ def _validate_fsh_files(
             deps,
             vs,
             cs,
+            fhir_version=fhir_version,
             verbose=verbose,
         )
 
@@ -421,7 +438,10 @@ def _validate_fsh_files(
 
 
 def validate_fsh(
-    fsh_filenames: List[FshPath], fname_validator: str, verbose: bool = False
+    fsh_filenames: List[FshPath],
+    fname_validator: str,
+    fhir_version: str,
+    verbose: bool = False,
 ) -> List[ValidatorStatus]:
     """
     Validate specific fsh files.
@@ -432,6 +452,7 @@ def validate_fsh(
 
     :param fsh_filename: FSH file names
     :param fname_validator: Full path to FHIR Java validator file
+    :param fhir_version: FHIR version to use in validator
     :param verbose: Print more information
     :return: List of validation status, full output and instance and profile names
     """
@@ -442,12 +463,17 @@ def validate_fsh(
         path_output=path_output,
         fnames=[f.absolute() for f in fsh_filenames],
         fname_validator=fname_validator,
+        fhir_version=fhir_version,
         verbose=verbose,
     )
 
 
 def validate_all_fsh(
-    base_path: str, subdir: str, fname_validator: str, verbose: bool = False
+    base_path: str,
+    subdir: str,
+    fname_validator: str,
+    fhir_version: str,
+    verbose: bool = False,
 ) -> List[ValidatorStatus]:
     """
     Validate all FSH files in a given subdir.
@@ -459,6 +485,7 @@ def validate_all_fsh(
     :param base_path: base path (of SUSHI project)
     :param subdir: subdirectory of profiles
     :param fname_validator: full path to FHIR Java validator file
+    :param fhir_version: FHIR version to use in validator
     :param verbose: Print more information
     :return: List of validation status, full output and instance and profile names
     """
@@ -477,6 +504,7 @@ def validate_all_fsh(
         path_output=path_output,
         fnames=fnames,
         fname_validator=fname_validator,
+        fhir_version=fhir_version,
         verbose=verbose,
     )
 
@@ -509,6 +537,7 @@ def run_validation(
     deps: Dict,
     vs: Dict,
     cs: Dict,
+    fhir_version: str,
     verbose: bool,
 ) -> List[ValidatorStatus]:
     """
@@ -522,19 +551,27 @@ def run_validation(
     :param deps: Dependencies from SUSHI output
     :param vs: ValueSets from SUSHI output
     :param cs: CodeSystems from SUSHI output
+    :param fhir_version: FHIR version to use in validator
     :param verbose: Print more information
     :return: List of validation result dicts containing validation status, full output and instance and profile names
     """
     cmd_base = [
         "java",
         f"-jar {fname_validator}",
-        "-version 4.0.1",
+        f"-version {fhir_version}",
         "-txLog logs/txlog.html",
     ]
     cmd_base += [f'-ig {dep["packageId"]}#{dep["version"]}' for dep in deps.values()]
 
     cmds = {}
 
+    # get questionnaire instances explicitly to include them -ig parameters (to be loaded by the validator)
+    questionnaires = [
+        i["filename"]
+        for i in instances.values()
+        if i["resourceType"] == "Questionnaire"
+    ]
+
     for fsh_instance in fsh_instances:
 
         if not fsh_instance["instance"] in instances:
@@ -547,6 +584,8 @@ def run_validation(
             cmd += [f'-ig {sdefs[instance["profile"]]["filename"]}']
         cmd += [f'-ig {valueset["filename"]}' for valueset in vs.values()]
         cmd += [f'-ig {codesystem["filename"]}' for codesystem in cs.values()]
+        # add all questionnaires that are not the instance itself
+        cmd += [f"-ig {qs}" for qs in questionnaires if qs != instance["filename"]]
         cmd += [f'-profile {instance["profile"]}', instance["filename"]]
 
         cmds[fsh_instance["instance"]] = cmd
@@ -555,7 +594,7 @@ def run_validation(
 
     for fsh_instance_name in cmds:
         print_box(
-            f'Validating {fsh_instance_name} against profile {instance["profile"]}'
+            f'Validating {fsh_instance_name} against profile {instances[fsh_instance_name]["profile"]}'
         )
         status = execute_validator(cmds[fsh_instance_name], verbose=verbose)
         status.instance = fsh_instance_name
@@ -682,6 +721,24 @@ def store_log(results: List[ValidatorStatus], log_path: Path) -> None:
     df.to_markdown(log_path / (log_basename + ".md"), index=False)
 
 
+def get_fhir_version_from_sushi_config(base_path: Path) -> str:
+    """
+    Get the FHIR version from the SUSHI config file.
+
+    :param base_path: Path to the SUSHI config file
+    :return: FHIR version string
+    """
+    conf_filename = base_path / "sushi-config.yaml"
+    if not conf_filename.exists():
+        raise FileNotFoundError(f"Could not find {conf_filename}")
+
+    with open(conf_filename, "r") as f:
+        conf = yaml.safe_load(f)
+        fhir_version = conf["fhirVersion"]
+
+    return fhir_version
+
+
 def assert_sushi_installed() -> None:
     """
     Assert that FSH Sushi is an executable on the system.
